@@ -5,7 +5,7 @@
  | program/include/iniset.php                                            |
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2014, The Roundcube Dev Team                       |
+ | Copyright (C) 2008-2015, The Roundcube Dev Team                       |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -21,7 +21,7 @@
 */
 
 // application constants
-define('RCMAIL_VERSION', '1.0.5');
+define('RCMAIL_VERSION', '1.1.1');
 define('RCMAIL_START', microtime(true));
 
 if (!defined('INSTALL_PATH')) {
@@ -55,8 +55,8 @@ if (set_include_path($include_path) === false) {
 @set_time_limit(120);
 
 // include composer autoloader (if available)
-if (@file_exists('vendor/autoload.php')) {
-    require 'vendor/autoload.php';
+if (@file_exists(INSTALL_PATH . 'vendor/autoload.php')) {
+    require INSTALL_PATH . 'vendor/autoload.php';
 }
 
 // include Roundcube Framework
@@ -68,6 +68,14 @@ spl_autoload_register('rcmail_autoload');
 // backward compatybility (to be removed)
 require_once INSTALL_PATH . 'program/include/bc.php';
 
+// load the UTF-8 portability layers from Patchwork
+// don't load mbstring layer as it conflicts with Roundcube Framework (#1490280)
+if (!function_exists('iconv')) {
+    \Patchwork\Utf8\Bootup::initIconv();
+}
+if (!function_exists('utf8_encode')) {
+    \Patchwork\Utf8\Bootup::initUtf8Encode();
+}
 
 /**
  * PHP5 autoloader routine for dynamic class loading
